module Hikki
  module Adapters
    class MemoryCollection < Hikki::Collection
      attr_reader :store, :uuid_generator

      def initialize(collection, store, uuid_generator)
        super(collection)
        @store = store
        @uuid_generator = uuid_generator
      end

      def index(field)
        store[index_key] ||= {}
        store[index_key][field.to_s] ||= {}
        true
      end

      def save(data)
        data = normalize_data(data)
        store[collection] ||= {}
        store[collection][data['id']] = data
        add_to_index(data)
        data
      end

      def find(id)
        store.fetch(collection, {}).fetch(id.to_s, {})
      end

      def all(options={})
        options = normalize_options(options)
        store.fetch(collection, {}).values[page_range(options)]
      end

      def find_by(field, value, options={})
        options = normalize_options(options)
        return find_by_index(field, value, options) if has_index?(field)
        all.select { |o| o.fetch(field.to_s) == value }[page_range(options)]
      end

      def remove(id)
        remove_from_index(find(id))
        store.fetch(collection, {}).delete(id.to_s)
        true
      end

      def remove_all
        store.delete(collection)
        store.delete(index_key)
        true
      end

      def has_index?(field)
        store.has_key?(index_key) &&
        store[index_key].has_key?(field.to_s)
      end

      def id_for(data)
        data.fetch('id', uuid_generator.uuid).to_s
      end

      private
      def index_key
        collection + '_index'
      end

      def find_by_index(field, value, options)
        store[index_key][field.to_s].fetch(value, []).map { |id| find(id) }.reject { |v| v == {} }[page_range(options)]
      end

      def add_to_index(data)
        store.fetch(index_key, {}).keys.each do |field|
          store[index_key][field][data[field]] ||= []
          store[index_key][field][data[field]] << data['id']
        end
      end

      def remove_from_index(data)
        return if data == {}
        store.fetch(index_key, {}).keys.each do |field|
          store[index_key][field].fetch(data[field], []).delete(data['id'])
        end
      end

    end
  end
end