module Hikki
  class Repository

    attr_reader :writers, :readers

    def initialize(writers=[], readers=[])
      @writers = Array(writers)
      @writers << Hikki::Adapters::MemoryAdapter.new if @writers.empty?
      @readers = Array(readers)
      @readers << @writers.first if @readers.empty?
    end

    def save(collection, data)
      head, *tail = *writers
      tail.reduce(head.save(collection, data)) { |result, writer| writer.save(collection, result) }
    end

    def index(collection, field)
      writers.all? { |writer| writer.index(collection, field) }
    end

    def remove(collection, field)
      writers.all? { |writer| writer.remove(collection, field) }
    end

    def remove_all(collection)
      writers.all? { |writer| writer.remove_all(collection) }
    end

    def find(collection, id)
      readers.each do |reader|
        result = reader.find(collection, id)
        return result unless result == {}
      end
      {}
    end

    def all(collection, options={})
      readers.each do |reader|
        result = reader.all(collection, options)
        return result unless result == []
      end
      []
    end

    def find_by(collection, field, value, options={})
      readers.each do |reader|
        result = reader.find_by(collection, field, value, options)
        return result unless result == []
      end
      []
    end

  end
end